@@ -24,6 +24,7 @@
 #include <locale.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
+#include <gdk/gdkx.h>
 
 #define MODEST_ACCOUNTS_GCONF_PATH "/apps/modest/accounts"
 #define MODEST_DEFAULT_ACCOUNT_GCONF_PATH "/apps/modest/default_account"
@@ -33,6 +34,7 @@
 #define TXTMSG_TO_MAIL_KEY_ACCOUNT GC_ROOT "/mail_account"
 #define TXTMSG_TO_MAIL_KEY_RECEIVER GC_ROOT "/mail_receiver"
 #define TXTMSG_SUBJECT_TEMPLATE_KEY GC_ROOT "/subject_template"
+#define TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY GC_ROOT "/subject_outgoing_template"
 #define TXTMSG_TO_MAIL_KEY_DISABLE_SMS_FORWARDING GC_ROOT "/disable_sms_forwarding"
 #define TXTMSG_TO_MAIL_KEY_ENABLE_CHAT_FORWARDING GC_ROOT "/enable_chat_forwarding"
 #define TXTMSG_TO_MAIL_KEY_DISABLE_INCOMING_FORWARDING GC_ROOT "/disable_incoming_forwarding"
@@ -53,6 +55,27 @@ _enable_disable_forwarding_option_box(HildonCheckButton* button, gpointer user_d
   gtk_widget_set_sensitive(GTK_WIDGET(user_data), hildon_check_button_get_active(button));
 }
 
+static void
+_size_changed(GdkScreen* screen,
+              gpointer user_data)
+{
+  GdkGeometry geometry;
+  if(gdk_screen_get_width(gdk_display_get_default_screen(gdk_display_get_default())) < 800)
+  {
+    geometry.min_height = 680;
+    geometry.min_width = 480; 
+  }
+  else
+  {
+    geometry.min_height = 360;
+    geometry.min_width = 800; 
+  }
+  gtk_window_set_geometry_hints(GTK_WINDOW(user_data),
+                                GTK_WIDGET(user_data),
+                                &geometry,
+                                GDK_HINT_MIN_SIZE);
+}
+
 static void
 _fill_account_list(GtkListStore* list_store, const gchar* active_account)
 {
@@ -138,6 +161,7 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
   gchar* receiver = gconf_client_get_string(gconf_client, TXTMSG_TO_MAIL_KEY_RECEIVER, NULL);
   gchar* account =  gconf_client_get_string(gconf_client, TXTMSG_TO_MAIL_KEY_ACCOUNT, NULL);
   gchar* subject_template = gconf_client_get_string(gconf_client, TXTMSG_SUBJECT_TEMPLATE_KEY, NULL);
+  gchar* subject_outgoing_template = gconf_client_get_string(gconf_client, TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY, NULL);
   GtkWidget* active_check_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
   gtk_button_set_label(GTK_BUTTON(active_check_button), "Enable Forwarding");
   hildon_check_button_set_active(HILDON_CHECK_BUTTON(active_check_button), forwarding_active);
@@ -177,17 +201,29 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
   hildon_gtk_entry_set_input_mode(GTK_ENTRY(to_entry), HILDON_GTK_INPUT_MODE_FULL);
   GtkWidget* subject_entry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT);
   hildon_gtk_entry_set_input_mode(GTK_ENTRY(subject_entry), HILDON_GTK_INPUT_MODE_FULL);
+  GtkWidget* subject_outgoing_entry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT);
+  hildon_gtk_entry_set_input_mode(GTK_ENTRY(subject_outgoing_entry), HILDON_GTK_INPUT_MODE_FULL);
   if(receiver != NULL)
     hildon_entry_set_text(HILDON_ENTRY(to_entry), receiver);
   if(subject_template != NULL && strlen(subject_template)!=0)
     hildon_entry_set_text(HILDON_ENTRY(subject_entry), subject_template);
   else
     hildon_gtk_entry_set_placeholder_text(GTK_ENTRY(subject_entry), "$(SENDER)");
-  
+  if(subject_outgoing_template != NULL && strlen(subject_outgoing_template)!=0)
+    hildon_entry_set_text(HILDON_ENTRY(subject_outgoing_entry), subject_outgoing_template);
+  else
+    hildon_gtk_entry_set_placeholder_text(GTK_ENTRY(subject_outgoing_entry), "$(RECEIVER)");
+
   GtkWidget* to_label = gtk_label_new(dgettext("modest", "mail_va_to"));
-  GtkWidget* subject_label = gtk_label_new(dgettext("modest", "mail_va_subject"));
+  gchar* subject_label_str = g_strdup_printf("%s<-", dgettext("modest", "mail_va_subject"));
+  GtkWidget* subject_label = gtk_label_new(subject_label_str);
+  g_free(subject_label_str);
+  subject_label_str = g_strdup_printf("%s->", dgettext("modest", "mail_va_subject"));
+  GtkWidget* subject_outgoing_label = gtk_label_new(subject_label_str);
+  g_free(subject_label_str);
   GtkWidget* to_box = gtk_hbox_new(FALSE, 0);
   GtkWidget* subject_box = gtk_hbox_new(FALSE, 0);
+  GtkWidget* subject_outgoing_box = gtk_hbox_new(FALSE, 0);
   GtkWidget* forwarding_option_box = gtk_vbox_new(FALSE, 0);
   GtkWidget* sms_chat_box = gtk_hbox_new(TRUE, 0);
   GtkWidget* incoming_outgoing_box = gtk_hbox_new(TRUE, 0);
@@ -197,6 +233,8 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
   gtk_box_pack_start(GTK_BOX(to_box), to_entry, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(subject_box), subject_label, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(subject_box), subject_entry, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(subject_outgoing_box), subject_outgoing_label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(subject_outgoing_box), subject_outgoing_entry, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(sms_chat_box), disable_sms_forwarding_button, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(sms_chat_box), enable_chat_forwarding_button, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(incoming_outgoing_box), disable_incoming_forwarding_button, TRUE, TRUE, 0);
@@ -215,9 +253,33 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
   gtk_box_pack_start(GTK_BOX(forwarding_option_box), account_picker_button, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(forwarding_option_box), to_box, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(forwarding_option_box), subject_box, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(forwarding_option_box), subject_outgoing_box, FALSE, FALSE, 0);
   gtk_widget_set_sensitive(forwarding_option_box, forwarding_active);
   gtk_box_pack_start(GTK_BOX(contents), forwarding_option_box, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), contents, TRUE, TRUE, 0);
+  GtkWidget* pan = hildon_pannable_area_new();
+  hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pan), contents);
+  g_object_set(pan, "hscrollbar-policy", GTK_POLICY_NEVER, NULL);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), pan, TRUE, TRUE, 0);
+  GdkGeometry geometry;
+  if(gdk_screen_get_width(gdk_display_get_default_screen(gdk_display_get_default())) < 800) 
+  { 
+    geometry.min_height = 680; 
+    geometry.min_width = 480; 
+  } 
+  else 
+  { 
+    geometry.min_height = 360;
+    geometry.min_width = 800; 
+  } 
+  gtk_window_set_geometry_hints(GTK_WINDOW(dialog), 
+                                dialog, 
+                                &geometry, 
+                                GDK_HINT_MIN_SIZE); 
+
+  g_signal_connect(gdk_display_get_default_screen(gdk_display_get_default()), 
+                   "size-changed", 
+                   G_CALLBACK(_size_changed), dialog); 
+
   gtk_widget_show_all(dialog);
   
   if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
@@ -252,6 +314,11 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
     {
       gconf_client_set_string(gconf_client, TXTMSG_SUBJECT_TEMPLATE_KEY, subject, NULL);
     }
+    const gchar* subject_outgoing = hildon_entry_get_text(HILDON_ENTRY(subject_outgoing_entry));
+    if(g_strcmp0(subject_outgoing, subject_outgoing_template)!=0)
+    {
+      gconf_client_set_string(gconf_client, TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY, subject_outgoing, NULL);
+    }
     GtkTreeIter iter;
     hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(account_selector),
 				       0, &iter);
@@ -276,6 +343,7 @@ osso_return_t execute(osso_context_t* osso, gpointer user_data, gboolean user_ac
   g_free(receiver);
   g_free(account);
   g_free(subject_template);
+  g_free(subject_outgoing_template);
   g_object_unref(gconf_client);
 
   return OSSO_OK;
