@@ -38,6 +38,7 @@
 #define TXTMSG_TO_MAIL_KEY_ACCOUNT GC_ROOT "/mail_account"
 #define TXTMSG_TO_MAIL_KEY_RECEIVER GC_ROOT "/mail_receiver"
 #define TXTMSG_SUBJECT_TEMPLATE_KEY GC_ROOT "/subject_template"
+#define TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY GC_ROOT "/subject_outgoing_template"
 #define TXTMSG_TO_MAIL_KEY_DISABLE_SMS_FORWARDING GC_ROOT "/disable_sms_forwarding"
 #define TXTMSG_TO_MAIL_KEY_ENABLE_CHAT_FORWARDING GC_ROOT "/enable_chat_forwarding"
 #define TXTMSG_TO_MAIL_KEY_DISABLE_INCOMING_FORWARDING GC_ROOT "/disable_incoming_forwarding"
@@ -54,6 +55,7 @@ struct _TxtMsgToMailStatusPluginPrivate
   gchar* mail_account;
   gchar* mail_receiver;
   gchar* subject_template;
+  gchar* subject_outgoing_template;
   guint gconf_notify_handler;
   RTComEl *eventlogger;
   DBusGConnection* session_bus;
@@ -62,7 +64,7 @@ struct _TxtMsgToMailStatusPluginPrivate
 HD_DEFINE_PLUGIN_MODULE(TxtMsgToMailStatusPlugin, txt_msg_to_mail_status_plugin, HD_TYPE_STATUS_MENU_ITEM);
 
 static void
-send_mail(const gchar* remote_uid, const gchar* message, TxtMsgToMailStatusPlugin* plugin)
+send_mail(const gchar* local_uid, const gchar* remote_uid, const gchar* message, TxtMsgToMailStatusPlugin* plugin, gint outgoing)
 {
   if(!plugin->priv->enabled ||
      plugin->priv->mail_sender == NULL ||
@@ -87,19 +89,38 @@ send_mail(const gchar* remote_uid, const gchar* message, TxtMsgToMailStatusPlugi
 
     GHashTable* text_info = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
     g_hash_table_insert(text_info,"plain-body", g_strdup(message));
-    
-    if(plugin->priv->subject_template == NULL || strlen(plugin->priv->subject_template)==0)
+
+    if((outgoing == 0) || (plugin->priv->subject_outgoing_template == 0 ||
+			   strlen(plugin->priv->subject_outgoing_template)==0))
     {
-      g_hash_table_insert(text_info, "subject", g_strdup(remote_uid));
+      if(plugin->priv->subject_template == NULL || strlen(plugin->priv->subject_template)==0)
+      {
+	g_hash_table_insert(text_info, "subject", g_strdup(remote_uid));
+      }
+      else
+      {
+	gchar** tile = g_strsplit(plugin->priv->subject_template, "$(SENDER)", -1);
+	gchar* subject = g_strjoinv(remote_uid, tile);
+	g_strfreev(tile);
+	tile = g_strsplit(subject, "$(RECEIVER)", -1);
+	gchar* subject_2 = g_strjoinv(local_uid, tile);
+	g_free(subject);
+	g_hash_table_insert(text_info, "subject", subject_2);
+	g_strfreev(tile);
+      }
     }
-    else
+    else if(outgoing == 1)
     {
-      gchar** tile = g_strsplit(plugin->priv->subject_template, "$(SENDER)", -1);
-      gchar* subject = g_strjoinv(remote_uid, tile);
-      g_hash_table_insert(text_info, "subject", subject);
+      gchar** tile = g_strsplit(plugin->priv->subject_outgoing_template, "$(SENDER)", -1);
+      gchar* subject = g_strjoinv(local_uid, tile);
+      g_strfreev(tile);
+      tile = g_strsplit(subject, "$(RECEIVER)", -1);
+      gchar* subject_2 = g_strjoinv(remote_uid, tile);
+      g_free(subject);
+      g_hash_table_insert(text_info, "subject", subject_2);
       g_strfreev(tile);
     }
-
+    
     GPtrArray* dummy1 = g_ptr_array_new();
     GPtrArray* dummy2 = g_ptr_array_new();
     GHashTable* dummy3 = g_hash_table_new(NULL, NULL);
@@ -160,7 +181,7 @@ _new_event_cb(RTComEl* backend,
 	if((outgoing == 0 && !plugin->priv->disable_incoming_forwarding) ||
 	   (outgoing == 1 && plugin->priv->enable_outgoing_forwarding))
 	{
-	  send_mail(remote_uid, text, plugin);
+	  send_mail(local_uid, remote_uid, text, plugin, outgoing);
 	}
       }
       g_free(text);
@@ -226,6 +247,14 @@ _gconf_settings_changed(GConfClient* client,
     if(gconf_value_get_string(value)!=NULL)
       plugin->priv->subject_template = g_strdup(gconf_value_get_string(value));
   }
+  if(g_strcmp0(gconf_entry_get_key(entry), TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY) == 0)
+  {
+    GConfValue* value = gconf_entry_get_value(entry);
+    if(plugin->priv->subject_outgoing_template)
+      g_free(plugin->priv->subject_outgoing_template);
+    if(gconf_value_get_string(value)!=NULL)
+      plugin->priv->subject_outgoing_template = g_strdup(gconf_value_get_string(value));
+  }
   if(g_strcmp0(gconf_entry_get_key(entry), TXTMSG_TO_MAIL_KEY_ACCOUNT) == 0)
   {
     GConfValue* value = gconf_entry_get_value(entry);
@@ -309,6 +338,10 @@ _read_gconf_settings(TxtMsgToMailStatusPlugin* plugin)
   {
     g_free(plugin->priv->subject_template);
   }
+  if(plugin->priv->subject_outgoing_template)
+  {
+    g_free(plugin->priv->subject_outgoing_template);
+  }
   if(plugin->priv->mail_account)
   {
     g_free(plugin->priv->mail_account);
@@ -317,6 +350,7 @@ _read_gconf_settings(TxtMsgToMailStatusPlugin* plugin)
   plugin->priv->mail_account = gconf_client_get_string(client, TXTMSG_TO_MAIL_KEY_ACCOUNT, NULL);
   plugin->priv->mail_receiver = gconf_client_get_string(client, TXTMSG_TO_MAIL_KEY_RECEIVER, NULL);
   plugin->priv->subject_template = gconf_client_get_string(client, TXTMSG_SUBJECT_TEMPLATE_KEY, NULL);
+  plugin->priv->subject_outgoing_template = gconf_client_get_string(client, TXTMSG_SUBJECT_OUTGOING_TEMPLATE_KEY, NULL);
   g_object_unref(client);
 }
 
@@ -347,6 +381,11 @@ txt_msg_to_mail_status_plugin_finalize(GObject* object)
     g_free(plugin->priv->subject_template);
     plugin->priv->subject_template = NULL;
   }
+  if(plugin->priv->subject_outgoing_template)
+  {
+    g_free(plugin->priv->subject_outgoing_template);
+    plugin->priv->subject_outgoing_template = NULL;
+  }
   if(plugin->priv->eventlogger)
     g_object_unref(plugin->priv->eventlogger);
   if(plugin->priv->session_bus)
@@ -374,6 +413,7 @@ txt_msg_to_mail_status_plugin_init(TxtMsgToMailStatusPlugin* plugin)
   plugin->priv->mail_account = NULL;
   plugin->priv->mail_receiver = NULL;
   plugin->priv->subject_template = NULL;
+  plugin->priv->subject_outgoing_template = NULL;
   plugin->priv->eventlogger = rtcom_el_new();
   plugin->priv->session_bus = hd_status_plugin_item_get_dbus_g_connection(HD_STATUS_PLUGIN_ITEM(&plugin->parent),
 									  DBUS_BUS_SESSION, 
